<?php
/**
 * Epic Review Init
 *
 * @author Jegtheme
 * @since 1.0.0
 * @package epic-review
 */

namespace Epic\Review;

use Epic\Review\Customizer\Customizer;
use Epic\Review\Data\Backend;
use Epic\Review\Data\Single;
use Epic\Review\Metabox\Metabox;
use Epic\Review\FrontendAjax;

/**
 * Class Init
 *
 * @package Epic\Review
 */
class Init {
	/**
	 * Init instance variable
	 *
	 * @var Init
	 */
	private static $instance;

	/**
	 * Init constructor.
	 */
	private function __construct() {
		$this->init();
		$this->hook();
	}

	/**
	 * Setup Classes
	 */
	private function init() {
		FrontendAjax::instance();
		Backend::instance();
		Customizer::instance();
		Single::instance();
		Metabox::instance();
	}

	/**
	 * Setup Hooks
	 */
	private function hook() {
		add_filter( 'jeg_register_elements', [ $this, 'register_element' ] );
		add_filter( 'jeg_add_image_size', [ $this, 'register_image_size' ] );
		add_filter( 'epic_plugin_list', [ $this, 'plugin_list' ] );

		add_action( 'wp_print_styles', [ $this, 'load_frontend_css' ] );
		add_action( 'admin_print_styles', [ $this, 'load_backend_css' ] );
		add_action( 'elementor/editor/after_enqueue_styles', [ $this, 'load_backend_css' ] );
		add_action( 'wp_enqueue_scripts', [ $this, 'load_script' ] );
		add_action( 'widgets_init', [ $this, 'register_widget' ] );
	}

	/**
	 * Init instance function
	 *
	 * @return Init
	 */
	public static function instance() {
		if ( null === static::$instance ) {
			static::$instance = new static();
		}

		return static::$instance;
	}

	/**
	 * Plugin List
	 *
	 * @param array $plugins Plugin List.
	 *
	 * @return array
	 */
	public function plugin_list( $plugins ) {
		$plugins[] = [
			'id'      => 25909360,
			'name'    => EPIC_REVIEW_NAME,
			'slug'    => EPIC_REVIEW,
			'version' => EPIC_REVIEW_VERSION,
			'option'  => 'epic_review_license',
			'file'    => EPIC_REVIEW_FILE,
		];

		return $plugins;
	}

	/**
	 * Load Frontend CSS
	 */
	public function load_frontend_css() {
		wp_enqueue_style( 'epic-review', EPIC_REVIEW_URL . '/assets/css/frontend.css', null, EPIC_REVIEW_VERSION );
		wp_enqueue_style( 'font-awesome', EPIC_DASHBOARD_URL . '/assets/fonts/font-awesome/font-awesome.css', null, EPIC_DASHBOARD_VERSION );
		wp_enqueue_style( 'epic-jegicon', EPIC_REVIEW_URL . '/assets/fonts/jegicon/jegicon.css', null, EPIC_REVIEW_VERSION );
	}

	/**
	 * Load Backend CSS
	 */
	public function load_backend_css() {
		wp_enqueue_style( 'epic-review', EPIC_REVIEW_URL . '/assets/css/backend.css', null, EPIC_REVIEW_VERSION );
	}

	/**
	 * Load Scripts
	 */
	public function load_script() {
		wp_enqueue_script( 'waypoint', EPIC_REVIEW_URL . '/assets/js/jquery.waypoints.js', ['jquery'], EPIC_REVIEW_VERSION, true );
		wp_enqueue_script( 'chosen', EPIC_REVIEW_URL . '/assets/js/chosen.jquery.js', ['jquery'], EPIC_REVIEW_VERSION, true );
		wp_enqueue_script( 'epic-review', EPIC_REVIEW_URL . '/assets/js/plugin.js', ['jquery'], EPIC_REVIEW_VERSION, true );
		wp_enqueue_script( 'lazysizes', EPIC_REVIEW_URL . '/assets/js/lazysizes.js', ['jquery'], EPIC_REVIEW_VERSION, true );
	}

	/**
	 * Register all elements
	 *
	 * @param array $elements : Array containing registered elements.
	 *
	 * @return array
	 */
	public function register_element( $elements ) {
		$items = [
			'Review_Post',
			'Review_Search',
		];

		foreach ( $items as $item ) {
			$item_key  = 'epic_' . strtolower( $item );
			$namespace = '\Epic\Review\Elements';

			$elements[ $item_key ] =
				[
					'option'    => $namespace . '\Options\\' . $item . '_Option',
					'view'      => $namespace . '\Views\\' . $item . '_View',
					'elementor' => $namespace . '\Elementor\\' . $item . '_Elementor',
				];
		}

		return $elements;
	}

	/**
	 * Register all widgets
	 */
	public function register_widget() {
		if ( ! defined( 'JEG_THEME_URL' ) ) {
			return;
		}

		register_widget( 'Epic\Review\Widget\Review_List_Widget' );
		register_widget( 'Epic\Review\Widget\Review_Single_Widget' );
	}

	/**
	 * Register all image size
	 *
	 * @param array $sizes : Array containing registered image sizes.
	 *
	 * @return mixed
	 */
	public function register_image_size( $sizes ) {
		$sizes['360x540']       = [ 360, 540, true, 1500 ];
		$sizes['360x480']       = [ 360, 480, true, 1300 ];
		$sizes['90x90']         = [ 90, 90, true, 1000 ];
		$sizes['370x370']       = [ 370, 370, true, 1000 ];
		$sizes['featured-750']  = [ 750, 0, true, 1000 ];
		$sizes['featured-1140'] = [ 1140, 0, true, 1000 ];
		$sizes['120x86']        = [ 120, 86, true, 715 ];
		$sizes['370x296']       = [ 370, 296, true, 800 ];
		$sizes['370x275']       = [ 370, 275, true, 800 ];
		$sizes['750x536']       = [ 750, 536, true, 715 ];
		$sizes['360x180']       = [ 360, 180, true, 500 ];
		$sizes['800x400']       = [ 800, 400, true, 500 ];
		$sizes['1140x570']      = [ 1140, 570, true, 500 ];

		return $sizes;
	}
}
