<?php
/**
 * Epic Review Frontend
 *
 * @author Jegtheme
 * @since 1.0.0
 * @package epic-review
 */

namespace Epic\Review\Data;

/**
 * Class ReviewData
 *
 * @package Epic\Review\Data
 */
class RatingData {
	/**
	 * Post ID
	 *
	 * @var int
	 */
	private static $id;

	/**
	 * Widget
	 *
	 * @var boolean
	 */
	private static $widget;

	/**
	 * Bar
	 *
	 * @var string|null
	 */
	private static $review_bar;

	/**
	 * Score
	 *
	 * @var string|null
	 */
	private static $review_score;

	/**
	 * RatingData constructor.
	 *
	 * @param int $post_id .
	 * @param string $rating_type .
	 */
	public function __construct( $post_id, $rating_type, $widget = false ) {
		self::$id     = $post_id;
		self::$widget = $widget;

		switch ( $rating_type ) {
			case 'point':
				self::$review_bar   = $this->point_bar();
				self::$review_score = $this->point_score();
				break;
			case 'star':
				self::$review_bar   = $this->star_bar();
				self::$review_score = $this->star_score();
				break;
			case 'percentage':
			default:
				self::$review_bar   = $this->percentage_bar();
				self::$review_score = $this->percentage_score();
				break;
		}
	}

	/**
	 * Bar point
	 *
	 * @return string|null
	 */
	public function point_bar() {
		$bar         = null;
		$bar_wrapper = null;
		$rating      = jeg_metabox( 'epic_review_metabox.rating', null, self::$id );

		if ( $rating ) {
			foreach ( $rating as $value ) {
				if ( ! empty( $value['label'] ) ) {
					$width = $value['value'] * 10;

					if ( $this->is_amp() ) {
						$score  = '<span class="reviewscore">' . esc_attr( $value['value'] ) . '</span>';
						$bar_bg = '<span style="width: ' . esc_attr( $width ) . '%" class="barbg"></span>';
					} else {
						$score  = '<span class="reviewscore">0</span>';
						$bar_bg = '<span data-width="' . esc_attr( $width ) . '" class="barbg"></span>';
					}

					$bar .=
						'<li>
                        <strong>' . esc_html( $value['label'] ) . '</strong>
                        ' . $score . '
                        <div class="jeg_reviewbar_wrap">
                            ' . $bar_bg . '
                        </div>
                    </li>';
				}
			}
		}

		if ( $bar ) {
			if ( self::$widget ) {
				$bar_wrapper =
					'<div class="jeg_reviewscore jeg_reviewbars clearfix" data-scoretype="point">
                    <ul>' . $bar . '</ul>
                </div>';
			} else {
				$bar_wrapper =
					'<div class="jeg_reviewscore jeg_reviewbars clearfix" data-scoretype="point">
                    <h3>' . esc_html__( 'Review Breakdown', 'epic-review' ) . '</h3>
                    <ul>' . $bar . '</ul>
                </div>';
			}
		}

		return $bar_wrapper;
	}

	/**
	 * Check if AMP
	 *
	 * @return bool
	 */
	private function is_amp() {
		$is_amp = false;
		if ( function_exists( 'is_amp_endpoint' ) ) {
			$is_amp = is_amp_endpoint();
		}

		return $is_amp;
	}

	/**
	 * Set Score Value
	 *
	 * @return string|null
	 */
	public function point_score() {
		$result = $this->is_rating_empty();

		if ( ! $result ) {
			$rating_value = get_post_meta( self::$id, 'epic_rating_mean', true );
			$rating_class = $this->get_rating_class( $rating_value );
			$rating_mean  = round( $rating_value, 1 );

			if ( self::$widget ) {
				$review_score_wrapper =
					'<div class="average_review_score ' . $rating_class . '">
                    <h3 class="average_score">' . esc_html__( 'Average Score', 'epic-review' ) . '</h3>
                    <div class="score_value">' . esc_html( $rating_mean ) . '</div>                    
                </div>';
			} else {
				$review_score_wrapper =
					'<div class="review_score ' . $rating_class . '">
                <span class="score_value">' . esc_html( $rating_mean ) . '</span>
                <span class="score_text">' . esc_html__( 'Score', 'epic-review' ) . '</span>
            </div>';
			}

			return $review_score_wrapper;
		}

		return null;
	}

	/**
	 * Check if rating is empty
	 *
	 * @return bool
	 */
	public function is_rating_empty() {
		$rating = jeg_metabox( 'epic_review_metabox.rating', null, self::$id );
		$result = true;

		if ( $rating ) {
			foreach ( $rating as $value ) {
				if ( ! empty( $value['label'] ) ) {
					$result = false;
				}
			}
		}

		return $result;
	}

	/**
	 * Get rating class
	 *
	 * @param int $score .
	 *
	 * @return string
	 */
	public function get_rating_class( $score ) {
		if ( $score >= 9 ) {
			return 'score_good';
		} elseif ( $score >= 8 && $score < 9 ) {
			return 'score_good';
		} elseif ( $score >= 6 && $score < 8 ) {
			return 'score_avg';
		} else {
			return 'score_bad';
		}
	}

	/**
	 * Review Star Bar
	 *
	 * @return string|null
	 */
	public function star_bar() {
		$rating             = jeg_metabox( 'epic_review_metabox.rating', null, self::$id );
		$review_bar         = null;
		$review_bar_wrapper = null;

		if ( $rating ) {

			foreach ( $rating as $value ) {
				if ( ! empty( $value['label'] ) ) {
					$rating_number = $value['value'];
					$rating_star   = '';

					for ( $i = 1; $i <= 5; $i ++ ) {
						if ( $rating_number >= 1.5 ) {
							$rating_star .= '<i class="fa fa-star"></i>';
						}

						if ( $rating_number >= 0.5 && $rating_number < 1.5 ) {
							$rating_star .= '<i class="fa fa-star-half-o"></i>';
						}

						if ( $rating_number < 0.5 ) {
							$rating_star .= '<i class="fa fa-star-o"></i>';
						}

						$rating_number = $rating_number - 2;
					}

					$review_bar .=
						'<li>
		                    <strong>' . esc_html( $value['label'] ) . '</strong>
		                    <div class="reviewscore">
		                        ' . $rating_star . '
		                    </div>
		                </li>';
				}
			}
		}

		if ( $review_bar ) {
			if ( self::$widget ) {
				$review_bar_wrapper =
					'<div class="jeg_reviewscore jeg_review_stars jeg_reviewbars clearfix">
                    <ul>' . $review_bar . '</ul>
                </div>';
			} else {
				$review_bar_wrapper =
					'<div class="jeg_reviewscore jeg_review_stars jeg_reviewbars clearfix">
                    <h3>' . esc_html__( 'Review Breakdown', 'epic-review' ) . '</h3>
                    <ul>' . $review_bar . '</ul>
                </div>';
			}
		}

		return $review_bar_wrapper;
	}

	/**
	 * Review Star Score
	 *
	 * @return string|null
	 */
	public function star_score() {
		$result = $this->is_rating_empty();

		if ( ! $result ) {
			$rating_value = get_post_meta( self::$id, 'epic_rating_mean', true );
			$rating_class = $this->get_rating_class( $rating_value );
			$rating_mean  = round( $rating_value / 2, 1 );

			if ( self::$widget ) {
				$review_score_wrapper =
					'<div class="average_review_score ' . $rating_class . '">
                    <h3 class="average_score">' . esc_html__( 'Average Score', 'epic-review' ) . '</h3>
                    <div class="score_value">' . esc_html( $rating_mean ) . '</div>               
                </div>';
			} else {
				$review_score_wrapper =
					'<div class="review_score ' . $rating_class . '">
	                <span class="score_value">' . esc_html( $rating_mean ) . '</span>
	                <span class="score_text">' . esc_html__( 'Score', 'epic-review' ) . '</span>
	            </div>';
			}

			return $review_score_wrapper;
		}

		return null;
	}

	/**
	 * Review Percentage Bar
	 *
	 * @return string|null
	 */
	public function percentage_bar() {
		$rating             = jeg_metabox( 'epic_review_metabox.rating', null, self::$id );
		$review_bar         = null;
		$review_bar_wrapper = null;
		$review_score       = null;
		$barbg              = null;

		if ( $rating ) {
			foreach ( $rating as $value ) {
				if ( ! empty( $value['label'] ) ) {
					$rating_number = $value['value'] * 10;

					if ( $this->is_amp() ) {
						$review_score = '<span class="reviewscore">' . esc_html( $rating_number ) . '%</span>';
						$barbg        = '<span style="width: ' . esc_attr( $rating_number ) . '%" class="barbg"></span>';
					} else {
						$review_score = '<span class="reviewscore">0%</span>';
						$barbg        = '<span data-width="' . esc_attr( $rating_number ) . '" class="barbg"></span>';
					}

					$review_bar .=
						'<li>
                        <strong>' . esc_html( $value['label'] ) . '</strong>
                        ' . $review_score . '
                        <div class="jeg_reviewbar_wrap">
                            ' . $barbg . '
                        </div>
                    </li>';
				}
			}
		}

		if ( $review_bar ) {
			if ( self::$widget ) {
				$review_bar_wrapper =
					'<div class="jeg_reviewscore jeg_reviewbars clearfix" data-scoretype="percent">
                    <ul>' . $review_bar . '</ul>
                </div>';
			} else {
				$review_bar_wrapper =
					'<div class="jeg_reviewscore jeg_reviewbars clearfix" data-scoretype="percent">
                    <h3>' . esc_html__( 'Review Breakdown', 'epic-review' ) . '</h3>
                    <ul>' . $review_bar . '</ul>
                </div>';
			}
		}

		return $review_bar_wrapper;
	}

	/**
	 * Review Percentage Score
	 *
	 * @return string|null
	 */
	public function percentage_score() {
		$result = $this->is_rating_empty();

		if ( ! $result ) {
			$rating_value = get_post_meta( self::$id, 'epic_rating_mean', true );
			$rating_class = $this->get_rating_class( $rating_value );
			$rating_mean  = round( $rating_value * 10, 0 );

			if ( self::$widget ) {
				$review_score_wrapper =
					'<div class="average_review_score ' . $rating_class . '">
                    <h3 class="average_score">' . esc_html__( 'Average Score', 'epic-review' ) . '</h3>
                    <div class="score_value">' . esc_html( $rating_mean ) . '<span class="percentsym">%</span> </div>                    
                </div>';
			} else {
				$review_score_wrapper =
					'<div class="review_score ' . $rating_class . '">
	                <span class="score_value">' . esc_html( $rating_mean ) . '<span class="percentsym">%</span> </span>
	                <span class="score_text">' . esc_html__( 'Score', 'epic-review' ) . '</span>
	            </div>';
			}

			return $review_score_wrapper;
		}

		return null;
	}

	/**
	 * Get Review Bar Value
	 *
	 * @return string|null
	 */
	public function get_review_bar() {
		return self::$review_bar;
	}

	/**
	 * Get Review Score Value
	 *
	 * @return string|null
	 */
	public function get_review_score() {
		return self::$review_score;
	}
}
