<?php


namespace Epic\Review\Widget;

use Epic\Review\Widget\Widget_Base;
use WP_Query;

/**
 * Class Review_List_Widget
 *
 * @package Epic\Review\Widget
 */
class Review_List_Widget extends Widget_Base {
	/**
	 * Review_List_Widget constructor.
	 */
	public function __construct() {
		parent::__construct(
			'epic_review_list',
			esc_html__( 'Epic - Review List', 'epic-review' ),
			[
				'description'                 => esc_html__( 'Review List for your website', 'epic-review' ),
				'customize_selective_refresh' => true,
			],
			null
		);
		add_filter( 'dynamic_sidebar_params', [ $this, 'widget_class' ], 10, 1 );
	}

	/**
	 * Register widget with WordPress.
	 *
	 * @param - $instance .
	 */
	public function form( $instance ) {
		$options = [
			'title' => [
				'title' => esc_html__( 'Title', 'epic-review' ),
				'desc'  => esc_html__( 'Title on widget header.', 'epic-review' ),
				'type'  => 'text',
				'group' => 'general-setting',
			],
			'limit' => [
				'title'   => esc_html__( 'Total limit post', 'epic-review' ),
				'desc'    => esc_html__( 'Set the number of post shown on widget.', 'epic-review' ),
				'type'    => 'slider',
				'options' => [
					'min'  => '2',
					'max'  => '15',
					'step' => '1',
				],
				'default' => 6,
				'group'   => 'general-setting',
			],
		];

		$this->render_widget_form( $instance, $options );
	}

	/**
	 * Front-end display of widget.
	 *
	 * @param array $args Widget arguments.
	 * @param array $instance Saved values from database.
	 */
	public function widget( $args, $instance ) {
		$title = apply_filters( 'widget_title', isset( $instance['title'] ) ? $instance['title'] : '' );

		sanitize_review( $args['before_widget'] );

		if ( ! empty( $title ) ) {
			sanitize_review( $args['before_title'] . esc_html( $title ) . $args['after_title'] );
		}

		$this->render_widget( $instance );

		sanitize_review( $args['after_widget'] );
	}

	/**
	 * Render widget content
	 *
	 * @param array $instance .
	 */
	public function render_widget( $instance ) {
		$output =
			'<div class="jeg_review_list_widget">' .
			$this->render_output( $instance ) .
			'</div>';

		sanitize_review( $output );
	}

	/**
	 * Render Output
	 *
	 * @param - $instance .
	 *
	 * @return string
	 */
	public function render_output( $instance ) {
		$review_html = '';
		$query       = new WP_Query(
			[
				'post_type'      => 'post',
				'post_status'    => [ 'publish' ],
				'posts_per_page' => $instance['limit'],
				'meta_query'     => [
					'relation' => 'AND',
					[
						'key'     => 'epic_review_metabox',
						'value'   => '"enable_review";s:1:"1"',
						'compare' => 'LIKE',
					],
				],
			]
		);

		add_filter( 'excerpt_length', [ $this, 'review_excerpt_length' ] );
		add_filter( 'excerpt_more', [ $this, 'review_excerpt_more' ] );

		if ( $query->have_posts() ) {
			while ( $query->have_posts() ) {
				$query->the_post();
				$review_html .= $this->review_list_item( $query->post );
			}
		}

		wp_reset_postdata();

		remove_filter( 'excerpt_length', [ $this, 'review_excerpt_length' ] );
		remove_filter( 'excerpt_more', [ $this, 'review_excerpt_more' ] );

		return $review_html;
	}

	/**
	 * Review List Item
	 *
	 * @param object $post .
	 *
	 * @return string
	 */
	public function review_list_item( $post ) {
		$title        = jeg_metabox( 'epic_review_metabox.name' );
		$permalink    = get_the_permalink( $post );
		$thumbnail    = apply_filters( 'jeg_image_thumbnail', $post->ID, 'jeg-120x86' );
		$review       = epic_review_generate_rating( null, $post->ID, '' );
		$lowest_price = get_post_meta( $post->ID, 'epic_price_lowest', true );
		$excerpt      = wp_trim_words( jeg_metabox( 'epic_review_metabox.summary' ), 11, '' );
		$price_html   = '';

		if ( $lowest_price ) {
			$price_html = '<a class="jeg_price_review_list" href="$permalink">' . epic_review_get_price( $lowest_price ) . '</a>';
		}

		$output =
			'<div class = "jeg_post jeg_pl_sm ' . $post->ID . '">
                <div class="jeg_thumb">
                	' . epic_edit_post( $post->ID ) . '
                    <a href="' . $permalink . '">' . $thumbnail . '</a>
                </div>
                <div class="jeg_postblock_content">
                    <h2 class="jeg_post_title"><a property="url" href="' . $permalink . '">' . $title . $price_html . '</a></h2>
                    <div class="jeg_review_list_rating">' . $review . '</div>
                </div>
            </div>';

		return $output;
	}


	/**
	 * Check the widget options before it's wrapper loads to add classes
	 *
	 * @param - $params .
	 *
	 * @return array
	 */
	public function widget_class( $params ) {
		return $this->add_widget_classes( $params, 'epic_review_list' );
	}
}
